# This is about as complex as the prefs can get. It includes an example
# use of every Swallow preference.

# This is a Python module; the syntax here is pure Python. If you need
# help with Python, please see the excellent documentation at
# www.python.org.

import socket
import os
import time

# Used in getSaveFolders below
def addIfNotPresent(list,item):
  if item not in list:
    list.append(item)

# Grabbed below. This function that you get if you don't set the
# preference. Note that this function blindly assumes that inbox,
# outbox, and trash exist because they're guaranteed to exist.  Your
# function shouldn't assume that any other mailboxes exist.
def myMailboxNamesSortFunc(listOfNames):
  listOfNames.remove("inbox")
  listOfNames.remove("outbox")
  listOfNames.remove("trash")
  listOfNames.sort()
  listOfNames.insert(0,"trash")
  listOfNames.insert(0,"outbox")
  listOfNames.insert(0,"inbox")
  return listOfNames



class prefs:
  # In this example, we have three places that we might be. Prefs
  # can stay the same regardless of location (human name probably
  # would) or change (mail server probably would). Locations defined
  # in locationPrefs below.

  def __init__(self):
    # On startup, try to figure out where we are. User can change
    # manually. Of course self.myLocation can also be set statically.
    self.myLocation=self.getDefaultLocation()
    return None

  # These next few are functions so that you can do whatver you want. These
  # are examples of what I do.

  # This is called on a mail object when the user clicks the Save button in
  # the overview or hits the "s" key there. Return a list of mailboxes to
  # save the mail to. It's also called on mail that has just been sent to
  # file it somewhere. (There can be different versions for incoming and
  # outgoing mail, see below.) Of course, you could just return a list of
  # mailbox names (return ["saved"]) if you didn't care about having them
  # vary. Default is None, which doesn't save the mail.

  def getSaveFolders(msg):
    # I like to archive my mail by month so always return something like mail-2000-06
    mbs=[time.strftime("mail-%Y-%m",time.localtime(time.time()))]
    # Sometimes also save in other folders
    if msg.getHeader("from","").lower().find("my-buddy@example.com")<>-1:
      addIfNotPresent(mbs,"my-buddy")
    if msg.getHeader("to","").lower().find("my-buddy@example.com")<>-1:
      addIfNotPresent(mbs,"my-buddy")
    return mbs

  def replyMunger(headers,bodyFile):
    if headers["subject"].lower().find("[some-mailing-list])<>-1:
      headers["cc"]="example@example.com"
      headers["reply-to"]="list@example.com"
    return None

  # If threading is not turned on, lines in the overview are sorted by
  # score then sent date. If threading is turned on, sibling messages
  # are sorted that way and message trees are sorted so that trees with
  # higher average scores come first.

  # If a POP session retrieves an email with a score higher than
  # noisyScore, doHighScore thing is called.


  def calculateScore(mailObj):
    # Mail from my buddy is gets score 10
    if mailObj.getHeader("from","").lower().find("my-buddy@example.com")<>-1:
      return 10
    # Mail to or cc-ed to me gets score 5
    if mailObj.getHeader("to","").lower().find("me@example.org")<>-1:
      return 5
    if mailObj.getHeader("cc","").lower().find("me@example.org")<>-1:
      return 5
    return 0

  # Called at startup to figure out which of the locations above we're
  # currently in. Return one of the strings from locations tuple above
  # or "" to have the program arbitrarily set to first location.
  def getDefaultLocation(self):
    # Note that this technique for determining the local IP
    # may or may not work, depending on the exact way your
    # networking is set up. That's inherent in TCP/IP and
    # isn't Python's fault. Parsing the output of ifconfig
    # is another useful technique.
    fqdn=socket.getfqdn()
    hostOnly=fqdn.split(".")[0]
    # If I use the FQDN here, I get back 127.0.0.1 but
    # I get the right answer if I give the host only.
    ip=socket.gethostbyname(hostOnly)

    if ip=="192.200.9.90":
      return "office"

    if ip=="10.10.10.10":
      return "home"

    # This is prolly specific to Linux but I suspect that a similar
    # thing could be done under most other Unixes, perhaps by opening
    # a pipe from ifconfig.
    if os.path.isfile("/proc/net/route"):
      f=open("/proc/net/route")
      l=f.readline()
      while l<>"":
        i=l.split()
        if i[0]=="ppp0":
          f.close()
          return "ppp"
        l=f.readline()
      f.close()
    # Give up
    return ""

  # "me" added automtically to addessbook
  # No automatic import written. Must edit here in prefs. Overridden
  # by swdb if preference swdbfile is set.
  addressbookData= \
    (
    # Nickaname, address. Nicknames may not have "@"s or
    # spaces. Nicknames entered in mail editor address
    # fields are expanded to the corresponding address.
    # The reverse is done in the overview.
    ("example", '"Example E. Example" <example@example.org>'),
    ("anotherexample", "Another Example <example@example.com>"),
    )

  # Any pref may be listed in either place (even though that doesn't make
  # much sense in some cases) locationPrefs is searched first.
  # XXX Check for duplicates?
  staticPrefs= \
    {
    "myMailAddress": "example@example.org",

    "myHumanName": "Swallow user",

    # Reply-to header in outbound mail, default is no reply-to.
    # It can be edited in the mail editor. Could be added or
    # changed by defaultReplyMunger and/or defaultForwardMunger
    # "replyTo": "otheraddress@example.org",

    # Grab function from above
    "calculateScore": calculateScore,

    # Temp directory to use, default as below
    "tempDir": os.path.expanduser("~/swallow/temp"),

    # Fully-qalified domain name, default is whatever
    # socket.getfqdn() returns
    "fqdn": "lawnboy.example.com",

    # Log file path beginning, default is ~/swallow/temp/swallowlog
    # (i.e. in the default temp directory). Use None to disable
    # saving the log. A new log file is created at each startup. The
    # name has -yyyymmdd-hhmmss appended to it. Swallow cleans up
    # files from the temp directory after a week.
    # "logFilePath": os.path.expanduser("~/swallow/temp/swallowlog"),

    # Integer, default 99. If calculateScore returns value >= this value
    # on new mail, doHighScoreThing callback executed.
    "noisyScore": 5,

    # Callback, default do nothing
    "doHighScoreThing": \
      lambda: os.system("/usr/bin/play /usr/share/sounds/phone.wav &"),

    # Tuple of color names, for sendable, draft, not sendable; defualt as below
    # "outboxColors": ("black","blue","red"),

    # Contains Maildir, default as below
    # "maildirParent": os.path.expanduser("~"),

    # Headers to show by default, default as below
    # "nonBoringHeaders": ["from","to","cc","bcc","subject","date","reply-to"],

    # Where to put outbox, saved mail folders, etc. Default as below
    # "userMailboxDirParent": os.path.expanduser("~/Mail"),

    # Could have different functions for inbound and outbound mailboxes
    # but I don't, so I grab the same function for both
    "getSaveFoldersInbound": getSaveFolders,
    "getSaveFoldersOutbound": getSaveFolders,

    # Could put the data right here but I prefer not to,
    # instead grab tuple above
    "addressbookData": addressbookData,

    # 1 or 0, default 0
    # "emptyTrashOnQuit": 0,

    # Fonts to use, defaults as below
    # "fontILike": "7x13",
    # "fixedWidthFontILike": "7x13",
    # "boldFontILike": "7x13bold",

    # Mail specified number of days old in specified mailbox will be moved to
    # trash mailbox on quit. Done after empty trash on quit if that's turned
    # on so you'll have a chance to get it back out. Tuple of tuples:
    # mailbox name, days old; default empty (don't do anything). If
    # emptyTrashOnQuit is set to 1, trash is emptied before mail is moved
    # to trash so you get an opportunity to move it back out.
    # "autoTrash": (("current",30),),

    # Where is ispell? Default as below
    # "ispellLoc": "/usr/bin/ispell",

    # Spelling checker highlights space runs. As a typographer
    # I deplore using two spaces after a period but lots of folks
    # do it. Set to 1 to only highlight runs of 3 or more spaces.
    # Default 0
    # "twoSpacesOK": 0,

    # Colors for text foreground for things highlighted by spelling
    # check. First is for unrecognized words that ispell has suggestions
    # for, second is for unrecognized words that ispell has no
    # suggestions for, third is for space runs.
    # Default as below
    # spellingCheckColors=("red","purple","yellow1"),

    # Colors (fg,bg) to highlight text search matches in mail viewer
    # Default as below
    # textSearchColors=("white","blue"),

    # Location of your signature file. Default None. None or "" mean
    # don't add signature. Often os.path.expanduser("~/.signature)
    # is what someone might want.
    "sigFile": None,

    # When user asks to print mail, text is piped to this command. Default
    # is "/bin/cat > /dev/null". You may want something more like item below.
    #"printCommand": "/usr/bin/lpr",

    # Command to pipe mail to. Invoked by key "|" in mail viewer window.
    # Default is None (i.e. do nothing). I use it to pipe the mail
    # to an external spam review helper. Note that unlike printCommand,
    # it's the raw mail that's sent to the pipe, not necessarily what's
    # being shown in the viewer.
    # "pipeCommand": os.path.expanduser("~/swallow/spamgrep.py"),

    # Setting this one to the name of a datafile to use enables the
    # more-complex version of the addressbook. See README.swdb for
    # the details.
    # "swdbFile": os.path.expanduser("~/swallow/addresses.txt"),

    # If mail is larger in bytes than this value, don't fetch it. Instead
    # inform user and allow it to be fetched later if desired. The default
    # is 10M. Note that the constant kNeverFetchBiggerThan in swallow.py
    # will override this value if it's smaller. There is no trivial way
    # to fetch all mail regardless of size. See the caution in the readme
    # about Swallow storing mail in memory.
    # Default 10M as below:
    # "askBeforeFetchingMailThisBig": 10*1024*1024,

    # Lines in viewer that describe MIME parts are highlighted
    # according to these settings. Defaults as below.
    # "mimeCommentaryFG": "white",
    # "mimeCommentaryBG": "black",

    # If mail is delivered to your Maildir by some other program, you can
    # set this item to "scan" so that checking for mail just inspects the
    # inbox. Default is "pop".
    # "mailCheckAction": "pop",

    # Width at which to wrap outgoing mail, default as below:
    "outboundMailWrapLen": 70,

    # Should Swallow sanity-check outbound mail? Could complain about
    # some legitimate mail. Default true.
    "sanityCheckOutboundMail": 1,

    # If present, function is called with two arguments: the first
    # is a dict of headers that the reply will have and the second
    # is a file (pointer is at the start) that contains the text of
    # the body. The function can modify them as you like to change
    # the default reply that's presented to the user in the editor.
    # Useful for changing reply-to, cc, adding random signatures,
    # etc. based on whom the reply is being sent to, subject line,
    # etc.
    # (Grabs function from above.)
    "defaultReplyMunger": replyMunger,

    # Just like defaultReplyMunger but fiddles with forwarded mail.
    "defaultForwardMunger": None,

    # Similarly, for new mail.
    "defaultNewMailMunger": None,

    # In the mail editor, the rewrap button re-wraps the user's
    # text to the appropriate margin. So that:
    #
    # > Hi, how is your new house?
    # > I remember that you said that
    # Send help!
    # The llamas have escaped!
    #
    # is wrapped more attractively as:
    #
    # > Hi, how is your new house? I remember that you said that
    # Send help! The llamas have escaped!
    #
    # But if you regularly start emails with a line like "Dear Bruce,"
    # the next line would be wrapped up on to that line, which is
    # probably not what you want. This preference controls special-
    # casing of such a line. If set to true (the default), if the
    # message's first line is short (<=30 characters) and ends with
    # any of comma, colon, dash, semi-colon, or bang, it's not rewrapped
    # unless the insertion point is in it or it's part of the selection.
    "specialCaseRewrapDearLine": 1,

    # If the mouse pointer is over a widget you're typing into, Swallow
    # can hide the pointer so that it's easier to see what you're typing.
    # The pointer is un-hidden as soon as you move the mouse. Default
    # off.
    "hideMousePointerWhileTyping": 0,

    # Should automatic checking for new mail be turned on at startup?
    # Default no.
    "autoStartCheckRegularly": 0,

    # Colors for foreground of quoted text in viewer (unquoted text
    # remains black). First color used for singly-quoted text, second
    # for doubly-quoted, etc. As many as you like allowed, last one
    # used for all deeper levels. Default None, don't color quotes.
    "viewerQuoteColors": ("navy","darkgreen","gray25"),

    # Preference for controlling mailbox sort order. Default is 0,
    # date ascending. Score calculated by calculateScore() is always
    # taken into account first.
    "defaultMailboxSort": 0

    # Function to sort mailbox names for overview. Default is
    # invox, outbox, trash, then all others sorted ASCIIbetically
    "mailboxNamesSortFunc": myMailboxNamesSortFunc,

    # When replying, if swdb used, should Swallow get the name from Swdb
    # or use the one in the original header?
    "useSWDBNameOnReply": 0,

    }

  # The one corresponding to myLocation (default set by function above, can be changed
  # by user) is searched before staticPrefs. Any item may be in either place.
  locationPrefs= \
    {
    "home":
      {
      # Serves only as the default for the next two
      "mailServer": "mail.example.org",

      # Default is mailServer above
      #"smtpServer": "mail.example.org",

      # Default is 25
      #"smtpPort": 25,

      # Default is mailServer above
      #"popServer": "mail.example.org",

      # Default is environment variable LOGNAME.
      "popUser": "testuser",

      # If listed here, is rot13-ed before use. (Tiny rot13.py utility included
      # for convenience.) Not for security, just to prevent casual glances from
      # revealing passwords. Default is None. None and "" mean ask each time.
      "popPass": "furrfu",

      # userpass or apop, default is userpass
      "popAuth": "apop",

      # 1 or 0, default is 1
      #"popDele": 1,

      # Default is 110
      "popPort": 1100,

      # Delay between checks if Check regularly turned on;
      # Seconds, default 300
      "checkDelay": 60,

      },

    "office":
      {
      "mailServer":  "mail.example.com",
      "popPass":     "cnffjbeq",
      "popAuth":     "userpass",
      "checkDelay":  120,
      },

    "ppp":
      {
      "smtpServer":  "smtp.example.com",
      "popServer":   "pop.example.com",
      "popUser":     "dialup-user",
      "popPass":     "",
      "popDele":     0,
      "checkDelay":  300,
      },
    }

